#
#   Calculate the x-section for p p -> Higgs  + X
#   and plot the distribution in pt and eta
#   Monte Carlo integration.
#
#   Authors: H. Jung, 
#            A. Bermudez Martinez, L.I. Estevez Banos, J. Lidrych, M. Mendizabal Morentin, 
#            S. Taheri Monfared, P. L.S. Connor, Q. Wang, H. Yang, R. Zlebcik 
#   

# Import what is necessary

from math import pi, sin, cos, log, log10, sqrt
from ROOT import gRandom, gStyle, TLorentzVector, TCanvas, TH1D

# The cross section of the Higgs

def sigma(m2):
    # calculate partonic x-section
    #  
    # calculate sigma0 (qq -> gamma^*)
    # double aem=1./137.;
    # result = 4.*pi*pi*aem/3.;
    #
    # calculate sigma0(gg->H) 
    # sigma0 = as^2/pi /576 * GF * sqrt(2)*\Delta(1.-tau) 
    aS = 0.1      # alphaS
    GF = 1.166e-5 # Fermi constant
    result = aS**2 * sqrt(2.) * GF /pi / 576. 
    return result 

# Function to get randomly 2D Gaussian distribution

def gauss2D(sigma):
    kT  = sigma * sqrt(-2*log(gRandom.Uniform()))
    phi = 2*pi * gRandom.Uniform()
    kx = kT*cos(phi)
    ky = kT*sin(phi)
    return kx, ky

# Get the four-momentum of the parton according to the parton density

s = 7000*7000 # center of mass energy
Eb = sqrt(s)/2 # Beam energy
def getpdf(q2):
    xmin = 0.00001
    xmax = 0.999
    # insert here your code
    
    weight = 1
    # end code

    pVec = TLorentzVector()
    pVec.SetXYZM(kx, ky, x*Eb, 0.)
    return pVec, weight

# Book the histograms

histo1  =  TH1D("x1","x1",100, -5, 0.)
histo2  =  TH1D("x2","x2",100, -5, 0.)
histo3  =  TH1D("kt1 ","kt1 ",100, 0, 10.)
histo4  =  TH1D("kt2 ","kt2 ",100, 0, 10.)
histo5  =  TH1D("pt ","pt ",50, 0, 10.)
histo6  =  TH1D("eta ","eta",50, -8, 8.)
histo7  =  TH1D("Mass ","mass",50, 60., 160.)

# Scale of the process and the mass window

q2 = 125**2
mass_min = 0.
mass_max = 200.

m_higgs = 125.
Gamma = 0.4


# Loop over MC events

sum0 = sum00 = 0
nacc = 0
npoints = 100000

for n1 in range(npoints):
    # this is using importance sampling
    # generate starting distribution in x and kt
    # for first incoming parton
    # calculate px,py,pz, E and kt2
  
    # generate starting distribution in x and kt
    # for second incoming parton
    # calculate px,py,pz, E and kt2
  
    # calculate the mass of the resulting system (pa+pb)^2
    # calculate the pdfs at x1,q2 and x2,q2, with q2=10000
  
    # calculate the cross section 
    # xsec=pdf(x1,q2)*pdf(x2,q2)*sigma*gev2nb
    # the pdf should be included in the weight for x
  
    # calculate the integral and its uncertainty
  
    # plot the distribution in x1,x2 versus log10(x)
    # which Jacobian is needed to get the proper distribution ?

    #insert here your code
    ff = 0
    # end insert 

    nacc += 1
    sum0  +=  ff
    sum00 +=  ff**2


# Normalization and evaluation of the error

sum0  /= npoints
sum00 /= npoints
sigma2 = sum00 - sum0*sum0 
error = sqrt(sigma2/npoints) 
print (" nr of events accepted: ", nacc )
print (" integral for Higgs xsection is [pb]: " , sum0 * 1000., " +/- " , error*1000.)

# Plotting of x1, x2 and kT1 and kT2

gStyle.SetOptStat(0) # get rid of statistics box
c = TCanvas()
# divide the canvas in 1 parts in x and 1 in y
c.Divide(2,2)
c.cd(1)
histo1.Scale(1./npoints, "width")
histo1.Draw()
c.cd(2)
histo2.Scale(1./npoints, "width")
histo2.Draw()
c.cd(3)
histo3.Draw()
c.cd(4)
histo4.Draw()
c.Draw()

# Draw the pT, eta and mass of the Higgs boson

c5 = TCanvas()
histo5.Draw()
c5.Draw()
c6 = TCanvas()
histo6.Draw()
c6.Draw()
c7 = TCanvas()
histo7.Draw()
c7.Draw()
